
    (*********************************************************************
     *  DCEQFilter.pas                                                   *
     *                                                                   *
     *  This unit is Part of the DC-Param EQ Filter v1.1                 *
     *                                                                   *
     *  author    : Milenko Mitrovic                                     *
     *  email     : dcoder@dsp-worx.de                                   *
     *  web       : http://dsp-worx.de                                   *
     *  date      : 24-07-2003                                           *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *  (C) 2003, 2004 Milenko Mitrovic <dcoder@dsp-worx.de>             *
     *                                                                   *
     *********************************************************************)

unit DCEQFilter;

{.$DEFINE DBG_CHANNEL_MAPPING}

interface

uses
  DirectShow9, BaseClass, ActiveX, Windows, MMSystem, PropAbout, PropDSP,
  CPUMeter, Registry, DSUtil, SysUtils, Classes, DCEQTypes, dspConst, dmoConst,
  DCParametricEQ;

type
  TDCEQFilter = class(TBCTransInPlaceFilter, ISpecifyPropertyPages, IPersist,
                      IMediaFilter, IDCEQ)
  private
    FCPUMeter: TCPUMeter;
    FThisInstance: integer;
    FStream: TDSStream;
    FEQ: TDCParamEQ;
    FSpeaker: DWORD;
    FChannelIndex: Integer;
    FMinRangeDB: integer;
    FMaxRangeDB: integer;
    FPreset: String;
    procedure LoadSettings;
    procedure SaveSettings;
    procedure SetupSpeakers(pmt: PAMMediaType);
  public
    // IBaseFilter
    function CheckConnect(dir: TPinDirection; Pin: IPin) : HRESULT; override;
    function CheckInputType(mtIn: PAMMediaType): HRESULT; override;
    constructor Create(ObjName: string; unk: IUnKnown; out hr: HRESULT);
    constructor CreateFromFactory(Factory: TBCClassFactory; const Controller: IUnknown); override;
    destructor Destroy; override;
    function Transform(Sample: IMediaSample): HRESULT; override;
    function BeginFlush: HRESULT; override;
    function NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult; override;  stdcall;
    // ISpecifyPropertyPages
    function GetPages(out pages: TCAGUID): HResult; stdcall;
    // IDCEQ
    function getEnabled(out Enabled: LONGBOOL): HRESULT; stdcall;
    function setEnabled(Enabled: LONGBOOL): HRESULT; stdcall;
    function getPreAmp(Channel: integer; out Amp: Single): HRESULT; stdcall;
    function setPreAmp(Channel: integer; Amp: Single): HRESULT; stdcall;
    function getGainDB(Band: integer; Channel: integer; out Value: Single): HRESULT; stdcall;
    function setGainDB(Band: integer; Channel: integer; Value: Single): HRESULT; stdcall;
    function getCPUUsage(out Usage: Double): HRESULT; stdcall;
    function getSpeaker(out Speaker: Cardinal): HRESULT; stdcall;
    function getMinRangeDB(out Range: integer): HRESULT; stdcall;
    function setMinRangeDB(Range: integer): HRESULT; stdcall;
    function getMaxRangeDB(out Range: integer): HRESULT; stdcall;
    function setMaxRangeDB(Range: integer): HRESULT; stdcall;
    function getChannelIndex(out Index: integer): HRESULT; stdcall;
    function setChannelIndex(Index: integer): HRESULT; stdcall;
    function getChannels(out Channels: Integer): HRESULT; stdcall;
    function getPreset(out Preset: PChar): HRESULT; stdcall;
    function setPreset(Preset: PChar): HRESULT; stdcall;
  end;

var
  InstanceCount: integer = 0;

implementation

constructor TDCEQFilter.CreateFromFactory(Factory: TBCClassFactory; const Controller: IUnknown);
var
  hr: HRESULT;
begin
  Create(Factory.Name, Controller, hr);
end;

constructor TDCEQFilter.Create(ObjName: string; unk: IUnknown; out hr: HRESULT);
begin
  inherited Create(ObjName, unk, CLSID_DCEQFilter, hr);
  fThisInstance := InterlockedIncrement(InstanceCount);
  fCPUMeter := TCPUMeter.Create;
  fEQ := TDCParamEQ.Create;

  FMinRangeDB := -12;
  FMaxRangeDB := 12;
  FChannelIndex := 0;
  FPreset := '';

  LoadSettings;
end;

destructor TDCEQFilter.Destroy;
begin
  SaveSettings;
  fEQ.Free;
  fCPUMeter.Free;
  fThisInstance := InterlockedDecrement(InstanceCount);
  inherited Destroy;
end;

function TDCEQFilter.GetPages(out pages: TCAGUID): HResult;
begin
  Pages.cElems := 2;
  Pages.pElems := CoTaskMemAlloc(sizeof(TGUID) * Pages.cElems);
  if (Pages.pElems = nil) then
  begin
    Result := E_OUTOFMEMORY;
    Exit;
  end;
  Pages.pElems^[0] := CLSID_DCEQFilter_PropDSP;
  Pages.pElems^[1] := CLSID_DCEQFilter_PropAbout;
  Result := S_OK;
end;

function TDCEQFilter.NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if IsEqualGuid(IID, IID_IDCEQ) or IsEqualGuid(IID, IPersist) or IsEqualGuid(IID, IID_IMediaFilter) or
     IsEqualGuid(IID, IID_ISpecifyPropertyPages) then
   begin
     if GetInterface(IID, Obj) then Result := S_OK
                               else Result := E_NOINTERFACE;
  end
  else Result := inherited NonDelegatingQueryInterface(iid, Obj);
end;

function TDCEQFilter.CheckConnect(dir: TPinDirection; Pin: IPin): HRESULT;
begin
  if (fThisInstance > 1) then
  begin
    Result := E_FAIL;
    Exit;
  end;

  Result := inherited CheckConnect(dir,Pin);
end;

function TDCEQFilter.CheckInputType(mtIn: PAMMediaType): HRESULT;
begin
  if not IsEqualGUID (mtIn^.majortype ,MEDIATYPE_Audio) then
  begin
    Result := VFW_E_INVALIDMEDIATYPE;
    Exit;
  end;

  if (not IsEqualGUID (mtIn^.subtype ,MEDIASUBTYPE_PCM)) and
     (not IsEqualGUID (mtIn^.subtype ,MEDIASUBTYPE_IEEE_FLOAT)) and
     (not IsEqualGUID (mtIn^.subtype ,MEDIASUBTYPE_NULL)) then // Null because of SoundForge
  begin
    Result := VFW_E_INVALIDSUBTYPE;
    Exit;
  end;

  if not IsEqualGUID (mtIn^.formattype,FORMAT_WaveFormatEx) then
  begin
    Result :=  VFW_E_TYPE_NOT_ACCEPTED;
    Exit;
  end;

  fStream.Frequency := PWaveFormatEx(mtIn^.pbFormat)^.nSamplesPerSec;
  fStream.Channels := PWaveFormatEx(mtIn^.pbFormat)^.nChannels;
  fStream.Bits := PWaveFormatEx(mtIn^.pbFormat)^.wBitsPerSample;

  if (fStream.Bits <> 32) and (fStream.Bits <> 24) and (fStream.Bits <> 16) and (fStream.Bits <> 8) then
  begin
    Result := VFW_E_TYPE_NOT_ACCEPTED;
    Exit;
  end;

  fStream.SPDIF := (PWaveFormatEx(mtIn^.pbFormat)^.wFormatTag = WAVE_FORMAT_DOLBY_AC3_SPDIF) or
                   (PWaveFormatEx(mtIn^.pbFormat)^.wFormatTag = WAVE_FORMAT_DOLBY_AC3);
  fStream.DTS :=   (PWaveFormatEx(mtIn^.pbFormat)^.wFormatTag = WAVE_FORMAT_DTS) or
                   (PWaveFormatEx(mtIn^.pbFormat)^.wFormatTag = WAVE_FORMAT_DVD_DTS);

  fStream.Float := IsEqualGUID (mtIn^.subtype ,MEDIASUBTYPE_IEEE_FLOAT) or (PWaveFormatEx(mtIn^.pbFormat)^.wFormatTag = WAVE_FORMAT_IEEE_FLOAT) or
     ((PWaveFormatExtensible(mtIn^.pbFormat)^.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE) and  IsEqualGUID(PWaveFormatExtensible(mtIn^.pbFormat)^.SubFormat,KSDATAFORMAT_SUBTYPE_IEEE_FLOAT));

  SetupSpeakers(mtIn);
  Result := S_OK;
end;

function TDCEQFilter.Transform(Sample: IMediaSample): HRESULT;
var
  pSrc : PByte;
  mt : PAMMediaType;
begin
  if Sample.GetMediaType(mt) = S_OK then
  begin
    Result := CheckInputType(mt);
    if Result <> S_OK then
    begin
      DeleteMediaType(mt);
      Exit;
    end;
  end;

  if not (fStream.SPDIF or fStream.DTS) then
  begin
    Sample.GetPointer(pSrc);
    fCPUMeter.StartMeasure;
    fEQ.Process(pSrc,Sample.GetActualDataLength,fStream.Frequency,fStream.Bits,fStream.Channels,fStream.Float, FSpeaker);
    fCPUMeter.StopMeasure;
  end;

  Result := S_OK;
end;

function TDCEQFilter.BeginFlush : HRESULT;
begin
  fEQ.Flush;
  Result := inherited BeginFlush;
end;

function TDCEQFilter.getEnabled(out Enabled: LONGBOOL): HRESULT;
begin
  Enabled := fEQ.Enabled;
  Result := S_OK;
end;

function TDCEQFilter.setEnabled(Enabled: LONGBOOL): HRESULT;
begin
  fEQ.Enabled := Enabled;
  Result := S_OK;
end;

function TDCEQFilter.getPreAmp(Channel: integer; out Amp: Single): HRESULT;
begin
  Amp := fEQ.PreAmp[Channel];
  Result := S_OK;
end;

function TDCEQFilter.setPreAmp(Channel: integer; Amp: Single): HRESULT;
begin
  fEQ.PreAmp[Channel] := Amp;
  Result := S_OK;
end;

function TDCEQFilter.getGainDB(Band: integer; Channel: integer; out Value: Single): HRESULT;
begin
  Value := fEQ.GainDB[Band,Channel];
  Result := S_OK;
end;

function TDCEQFilter.setGainDB(Band: integer; Channel: integer; Value: Single): HRESULT;
begin
  fEQ.GainDB[Band,Channel] := value;
  Result := S_OK;
end;

function TDCEQFilter.getCPUUsage(out Usage: Double): HRESULT;
begin
  Usage := fCPUMeter.Usage;
  Result := S_OK;
end;

procedure TDCEQFilter.LoadSettings;
var
  Reg: TRegistry;
  i, c: integer;
begin
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_CURRENT_USER;
  if Reg.KeyExists('SOFTWARE\DSP-worx\DC-ParamEQ Filter') then
  begin
    if Reg.OpenKey('SOFTWARE\DSP-worx\DC-ParamEQ Filter', False) then
    begin
      if Reg.ValueExists('Enabled') then
        fEQ.Enabled := Reg.ReadBool('Enabled');

      for i := 0 to EQ_NUM_BANDS -1 do
        for c := 0 to MaxChannels -1 do
          if Reg.ValueExists('eq' + inttostr(i) + 'c' + inttostr(c)) then
            fEQ.GainDB[i,c] := Reg.ReadFloat('eq' + inttostr(i) + 'c' + inttostr(c));

      for i := 0 to MaxChannels -1 do
        if Reg.ValueExists('amp' + inttostr(i)) then
            fEQ.PreAmp[i] := Reg.ReadFloat('amp' + inttostr(i));

      if Reg.ValueExists('minrangedb')
        then FMinRangeDB := Reg.ReadInteger('minrangedb');

      if Reg.ValueExists('maxrangedb')
        then FMaxRangeDB := Reg.ReadInteger('maxrangedb');

      if Reg.ValueExists('channelindex')
        then FChannelIndex := Reg.ReadInteger('channelindex');

      if Reg.ValueExists('preset')
        then FPreset := Reg.ReadString('preset');
    end;
  end;
  Reg.Free;
end;

procedure TDCEQFilter.SaveSettings;
var
  Reg: TRegistry;
  i, c: integer;
begin
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_CURRENT_USER;
  if Reg.OpenKey('SOFTWARE\DSP-worx\DC-ParamEQ Filter', True) then
  begin
    Reg.WriteBool('Enabled', fEQ.Enabled);

    for i := 0 to EQ_NUM_BANDS -1 do
      for c := 0 to MaxChannels -1 do
        Reg.WriteFloat('eq' + inttostr(i) + 'c' + inttostr(c), fEQ.GainDB[i,c]);

    for i := 0 to MaxChannels -1 do
      Reg.WriteFloat('amp' + inttostr(i), fEQ.PreAmp[i]);

    Reg.WriteInteger('minrangedb', FMinRangeDB);
    Reg.WriteInteger('maxrangedb', FMaxRangeDB);
    Reg.WriteInteger('channelindex', FChannelIndex);
    Reg.WriteString('preset', FPreset);
  end;
  Reg.Free;
end;

procedure TDCEQFilter.SetupSpeakers(pmt: PAMMediaType);
{$IFDEF DBG_CHANNEL_MAPPING}
var
  str: String;
{$ENDIF}
begin
  if (PWaveFormatExtensible(pmt^.pbFormat)^.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE) then
  begin
    FSpeaker := PWaveFormatExtensible(pmt^.pbFormat)^.dwChannelMask;
  end else
  if (FStream.Channels = 2) then
  begin
    FSpeaker := SPEAKER_FRONT_LEFT or SPEAKER_FRONT_RIGHT;
  end else
  begin
    FSpeaker := SPEAKER_FRONT_CENTER;
  end;
{$IFDEF DBG_CHANNEL_MAPPING}
  str := '';
  if ((FSpeaker and SPEAKER_FRONT_LEFT) = SPEAKER_FRONT_LEFT) then str := str + ' SPEAKER_FRONT_LEFT';
  if ((FSpeaker and SPEAKER_FRONT_RIGHT) = SPEAKER_FRONT_RIGHT) then str := str + ' SPEAKER_FRONT_RIGHT';
  if ((FSpeaker and SPEAKER_FRONT_CENTER) = SPEAKER_FRONT_CENTER) then str := str + ' SPEAKER_FRONT_CENTER';
  if ((FSpeaker and SPEAKER_LOW_FREQUENCY) = SPEAKER_LOW_FREQUENCY) then str := str + ' SPEAKER_LOW_FREQUENCY';
  if ((FSpeaker and SPEAKER_BACK_LEFT) = SPEAKER_BACK_LEFT) then str := str + ' SPEAKER_BACK_LEFT';
  if ((FSpeaker and SPEAKER_BACK_RIGHT) = SPEAKER_BACK_RIGHT) then str := str + ' SPEAKER_BACK_RIGHT';
  if ((FSpeaker and SPEAKER_FRONT_LEFT_OF_CENTER) = SPEAKER_FRONT_LEFT_OF_CENTER) then str := str + ' SPEAKER_FRONT_LEFT_OF_CENTER';
  if ((FSpeaker and SPEAKER_FRONT_RIGHT_OF_CENTER) = SPEAKER_FRONT_RIGHT_OF_CENTER) then str := str + ' SPEAKER_FRONT_RIGHT_OF_CENTER';
  if ((FSpeaker and SPEAKER_BACK_CENTER) = SPEAKER_BACK_CENTER) then str := str + ' SPEAKER_BACK_CENTER';
  if ((FSpeaker and SPEAKER_SIDE_LEFT) = SPEAKER_SIDE_LEFT) then str := str + ' SPEAKER_SIDE_LEFT';
  if ((FSpeaker and SPEAKER_SIDE_RIGHT) = SPEAKER_SIDE_RIGHT) then str := str + ' SPEAKER_SIDE_RIGHT';
  if ((FSpeaker and SPEAKER_TOP_CENTER) = SPEAKER_TOP_CENTER) then str := str + ' SPEAKER_TOP_CENTER';
  if ((FSpeaker and SPEAKER_TOP_FRONT_LEFT) = SPEAKER_TOP_FRONT_LEFT) then str := str + ' SPEAKER_TOP_FRONT_LEFT';
  if ((FSpeaker and SPEAKER_TOP_FRONT_CENTER) = SPEAKER_TOP_FRONT_CENTER) then str := str + ' SPEAKER_TOP_FRONT_CENTER';
  if ((FSpeaker and SPEAKER_TOP_FRONT_RIGHT) = SPEAKER_TOP_FRONT_RIGHT) then str := str + ' SPEAKER_TOP_FRONT_RIGHT';
  if ((FSpeaker and SPEAKER_TOP_BACK_LEFT) = SPEAKER_TOP_BACK_LEFT) then str := str + ' SPEAKER_TOP_BACK_LEFT';
  if ((FSpeaker and SPEAKER_TOP_BACK_CENTER) = SPEAKER_TOP_BACK_CENTER) then str := str + ' SPEAKER_TOP_BACK_CENTER';
  if ((FSpeaker and SPEAKER_TOP_BACK_RIGHT) = SPEAKER_TOP_BACK_RIGHT) then str := str + ' SPEAKER_TOP_BACK_RIGHT';

// http://download.microsoft.com/download/winmediatech40/Utility/1.0/W98NT42KMeXP/EN-US/6channel.exe
// http://download.microsoft.com/download/winmediatech40/Utility/1.0/W98NT42KMeXP/EN-US/8channel.exe

// -- 8 Channel WMA
// SPEAKER_FRONT_LEFT
// SPEAKER_FRONT_RIGHT
// SPEAKER_FRONT_CENTER
// SPEAKER_LOW_FREQUENCY
// SPEAKER_BACK_LEFT
// SPEAKER_BACK_RIGHT
// SPEAKER_FRONT_LEFT_OF_CENTER
// SPEAKER_FRONT_RIGHT_OF_CENTER

// -- 8 Channel WAV
// SPEAKER_FRONT_LEFT
// SPEAKER_FRONT_RIGHT
// SPEAKER_FRONT_CENTER
// SPEAKER_LOW_FREQUENCY
// SPEAKER_BACK_LEFT
// SPEAKER_BACK_RIGHT

// -- 6 Channel WMA
// SPEAKER_FRONT_LEFT
// SPEAKER_FRONT_RIGHT
// SPEAKER_FRONT_CENTER
// SPEAKER_LOW_FREQUENCY
// SPEAKER_BACK_LEFT
// SPEAKER_BACK_RIGHT

// -- 6 Channel WAV
// SPEAKER_FRONT_LEFT
// SPEAKER_FRONT_RIGHT
// SPEAKER_FRONT_CENTER
// SPEAKER_LOW_FREQUENCY
// SPEAKER_BACK_LEFT
// SPEAKER_BACK_RIGHT


  OutputDebugString(pchar('Speakers: ' + str));
{$ENDIF}
end;

function TDCEQFilter.getSpeaker(out Speaker: Cardinal): HRESULT;
begin
  Result := S_OK;
  Speaker := FSpeaker;
end;

function TDCEQFilter.getMinRangeDB(out Range: integer): HRESULT;
begin
  Range := FMinRangeDB;
  Result := S_OK;
end;

function TDCEQFilter.setMinRangeDB(Range: integer): HRESULT;
begin
  FMinRangeDB := Range;
  Result := S_OK;
end;

function TDCEQFilter.getMaxRangeDB(out Range: integer): HRESULT;
begin
  Range := FMaxRangeDB;
  Result := S_OK;
end;

function TDCEQFilter.setMaxRangeDB(Range: integer): HRESULT;
begin
  FMaxRangeDB := Range;
  Result := S_OK;
end;

function TDCEQFilter.getChannelIndex(out Index: integer): HRESULT;
begin
  Result := S_OK;
  Index := FChannelIndex;
end;

function TDCEQFilter.setChannelIndex(Index: integer): HRESULT;
begin
  Result := S_OK;
  FChannelIndex := Index;
end;

function TDCEQFilter.getChannels(out Channels: Integer): HRESULT;
begin
  Channels := FStream.Channels;
  Result := S_OK;
end;

function TDCEQFilter.getPreset(out Preset: PChar): HRESULT;
begin
  Preset := PChar(FPreset);
  Result := S_OK;
end;

function TDCEQFilter.setPreset(Preset: PChar): HRESULT;
begin
  FPreset := Preset;
  Result := S_OK;
end;


initialization
  TBCClassFactory.CreateFilter(TDCEQFilter, StringToOleStr('DC-Graphic EQ Filter'), CLSID_DCEQFilter,
    CLSID_LegacyAmFilterCategory, MERIT_PREFERRED + 2, 2, @SudPins);

end.
